/*
 * Copyright (c) Members of the EGEE Collaboration. 2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors: Zsolt Molnar, CERN
 */

#include <string.h>
#include <stdlib.h>

#include "gfal_is_ifce_internal.h"

char *gfal_is_ifce_replace_substr(
    char const * const original, 
    char const * const pattern, 
    char const * const replacement) 
{
    size_t const replen = strlen(replacement);
    size_t const patlen = strlen(pattern);
    size_t const orilen = strlen(original);

    size_t patcnt = 0;
    const char * oriptr;
    const char * patloc;

    /* find how many times the pattern occurs in the original string */
    for (oriptr = original; 
         (patloc = strstr(oriptr, pattern)); 
         oriptr = patloc + patlen)
    {
        patcnt++;
    }

    /* allocate memory for the new string */
    size_t const retlen = orilen + patcnt * (replen - patlen);
    char * const returned = (char *) malloc( sizeof(char) * (retlen + 1) );

    if (returned != NULL)
    {
        /* copy the original string, replacing all the instances of the pattern */
        char * retptr = returned;

        for (oriptr = original; 
             (patloc = strstr(oriptr, pattern)); 
             oriptr = patloc + patlen)
        {
            size_t const skplen = patloc - oriptr;
            /* copy the section until the occurence of the pattern */
            strncpy(retptr, oriptr, skplen);
            retptr += skplen;
            /* copy the replacement */
            strncpy(retptr, replacement, replen);
            retptr += replen;
        }
        /* copy the rest of the string */
        strcpy(retptr, oriptr);
    }
    return returned;
}

